#include <stdio.h>
#include <string.h>
#include <limits.h>
#include <stdlib.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <unistd.h>
#include <dirent.h>

#include "common.h"

char *extract (char *src, char *variable);

/* ----------------------------------------------------------------------- */

int cgiMain (void)
{
    char  buf1[2048];       // temp buffers
    char  *p, *readme;
    int   i, n;
    unsigned long total_size = 0;
    DIR   *d;
    struct tm tm1;
    
    // data about file/directories
    int               N, nfiles, ndirs;
    struct file   *f, F;

    // get the name of the directory to process and various flags.
    get_options ();
    if (load_database ()) error ("Can't load list of files");

    cgiHeaderContentType ("text/html");
    fprintf (cgiOut, HTML_HEADER, options.path);
    print_file (HEADER);

    // check for READMEs
    p = make_absolute (options.path);
    d = opendir (p);
    while ((readme = next_readme (d)) != NULL)
    {
        fprintf (cgiOut, "<p><hr>\n<b>%s</b>:\n<pre>", readme);
        strcpy (buf1, options.path);
        strcats (buf1, readme);
        print_long_file (buf1, 16);
        fprintf (cgiOut, "</pre>\n");
    }
    closedir (d);
    
    // put out header with directory name
    print_directory_name (options.path, dir_description (options.path));

    sprintf (buf1, AUTOINDEX"?%s", put_options (SKIP_SORT_TYPE));
    print_sort_urls (buf1);
    
    // compile list of files/directories with attributes
    f = (struct file *) malloc (MAX_ENTRIES * sizeof (struct file));
    N = 0;
    for (i=0; i<fb.N && N<MAX_ENTRIES-1; i++)
    {
        if (!strcmp (fb.files[i].dir, options.path))
            f[N++] = fb.files[i];
    }
    qsort ((void *)f, N, sizeof (struct file), filecompare);

    // count directories and files
    ndirs = 0;
    nfiles = 0;
    for (i=0; i<N; i++)
    {
        switch (f[i].type)
        {
        case TYPE_DIRECTORY:
            ndirs++;
            break;
        case TYPE_REGULAR:
            nfiles++;
            break;
        case TYPE_SYMLINK:
            n = resolve_link (f[i]);
            if (n < 0) continue;
            if (fb.files[n].type == TYPE_DIRECTORY) ndirs++;
            if (fb.files[n].type == TYPE_REGULAR) nfiles++;
            break;
        }
    }

    // put multi-download link if enough files
    if (nfiles > 10)
        if (options.group_selection)
            fprintf (cgiOut, "\n<br><a href=\""AUTOINDEX"?%s&msel=0\">"
                    "Exit multi-download mode</a>",
                    put_options (SKIP_GROUPSEL));
        else
            fprintf (cgiOut, "\n<br><a href=\""AUTOINDEX"?%s&msel=1\">"
                    "Enter multi-download mode</a>",
                    put_options (SKIP_GROUPSEL));
    
    // process directories
    if (ndirs == 0)
    {
        fprintf (cgiOut, "\n\n<hr>\n"
                "<b>No subdirectories</b><br>\n\n");
        goto skip_to_files;
    }
    
    fprintf (cgiOut, "\n\n<hr>\n"
            "<b>%d subdirectories</b>"
            "\n"
            "<pre>", ndirs);

    for (i=0; i<N; i++)
    {
        switch (f[i].type)
        {
        case TYPE_DIRECTORY:
            F = f[i];
            break;
        case TYPE_SYMLINK:
            n = resolve_link (f[i]);
            if (n < 0 || fb.files[n].type != TYPE_DIRECTORY) continue;
            F = fb.files[n];
            if (F.desc == NULL)
            {
                sprintf (buf1, "-&gt; %s", f[i].desc);
                F.desc = chunk_put (buf1);
            }
            break;
        default:
            continue;
        }
        strcpy (buf1, F.dir);
        strcats (buf1, F.name);

        tm1 = *localtime (&(F.age));
        fprintf (cgiOut, "\n%4d %3s %2d %02d:%02d %9ld "
                "<a href=\""AUTOINDEX"?%s&path=%s\">%s</a>",
                1900+tm1.tm_year, month[tm1.tm_mon],
                tm1.tm_mday, tm1.tm_hour, tm1.tm_min, F.size,
                put_options (SKIP_PATH),
                hexify (buf1), f[i].name);
        if (F.hs)
            fprintf (cgiOut, "&nbsp;<a href=\""AUTOTREE"?%s&path=%s\">"SIGN_EXPAND"</a>",
                    put_options (SKIP_PATH), hexify (buf1));
        if (F.desc != NULL) 
            fprintf (cgiOut, "\n                            <i>%s</i>",
                     F.desc);
                
    }
    
    fprintf (cgiOut, "\n</pre>\n");
    
    // process files
skip_to_files:
    if (nfiles == 0)
    {
        fprintf (cgiOut, "\n<b>No files</b><br>\n");
        goto finalize;
    }

    for (i=0; i<N; i++)
        if (f[i].type == TYPE_REGULAR)
            total_size += f[i].size;

    fprintf (cgiOut, "\n<b>%d files (%s bytes)</b>",
            nfiles, insert_commas (total_size));

    // print download form
    if (options.group_selection)
        fprintf (cgiOut, "<form action=\""AUTODOWNLOAD1"\" method=post>");
    fprintf (cgiOut, "<pre>");
    print_list (f, N, LIST_SINGLE_DIR);
    fprintf (cgiOut, "\n</pre>");
    if (options.group_selection)
    {
        put_options_form (0);
        fprintf (cgiOut, "\n<input type=submit value=Download>"
               "\n</form>");
    }

    if (nfiles > 0)
        if (options.group_selection)
            fprintf (cgiOut, "\n<a href=\""AUTOINDEX"?%s&msel=0\">"
                    "Exit multi-download mode</a>",
                    put_options (SKIP_GROUPSEL));
        else
            fprintf (cgiOut, "\n<a href=\""AUTOINDEX"?%s&msel=1\">"
                    "Enter multi-download mode</a>",
                    put_options (SKIP_GROUPSEL));

finalize:
    print_footer ();
    print_file (FOOTER);
    print_file (COPYRIGHT);
    fprintf (cgiOut, HTTP_FOOTER);
    
    return 0;
}

