#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "common.h"

/*--------------------------------------------------------------------*/

void list_zip (char *path, char *name);
void list_arj (char *path, char *name);
void list_tar (char *path, char *name);
int  is_fileline (char *s);

/*--------------------------------------------------------------------*/

int cgiMain (void)
{
    char  filename[2048], filepath[4096];

    get_options ();
    
    // extract parameters
    cgiFormStringNoNewlines ("file", filename, sizeof (filename));
    if (filename[0] == '\0')
        error ("file to view isn't specified");
    
    cgiHeaderContentType ("text/html");
    
    cgiFormStringNoNewlines ("fpath", filepath, sizeof (filepath));
    if (filepath[0] == '\0') strcpy (filepath, "/");
    
    // output HTTP and HTML headers which are always needed
    fprintf (cgiOut, HTML_HEADER, filename);
    print_file (HEADER);

    fprintf (cgiOut, "<h2>Contents of %s/%s</h2>\n<hr>\n", filepath, filename);
    
    // analyze filename and invoke corresponding processor
    if (mask_matchi ("*.zip", filename) ||
        mask_matchi ("*.exe", filename))
        list_zip (filepath, filename);
    
    else if (mask_matchi ("*.tar.gz", filename) ||
             mask_matchi ("*.tar.z",  filename) ||
             mask_matchi ("*.tgz",    filename) ||
             mask_matchi ("*.taz",    filename) ||
             mask_matchi ("*.tar",    filename))
        list_tar (filepath, filename);
                          
    else if (mask_matchi ("*.arj", filename) ||
             mask_matchi ("*.a0?", filename))
        list_arj (filepath, filename);

    else
        fprintf (cgiOut, "Sorry, this archive format isn't supported yet\n");

    print_footer ();
    print_file (FOOTER);
    print_file (COPYRIGHT);
    fprintf (cgiOut, HTTP_FOOTER);
    
    return 0;
}

/*--------------------------------------------------------------------*/

void list_zip (char *path, char *name)
{
    int   l, inside_comment;
    char  buf[4096], buf1[4096], tempfile1[64];
    FILE  *fp;
    struct file f;
    struct tm tm1;

    sprintf (tempfile1, TEMPDIR"contents.%d", getpid ());
    
    // create listing by executing unzip
    sprintf (buf, "unzip -Z -z -m %s/%s >%s",
             make_absolute (path), name, tempfile1);
    cgiSaferSystem (buf);

    // read and parse listing
    fp = fopen (tempfile1, "r");
    fgets (buf, sizeof(buf), fp); // skip first line; it's meaningless

    inside_comment = 0;
    fprintf (cgiOut, "\n<pre>\n");
    while (fgets (buf, sizeof(buf), fp) != NULL)
    {
        strip_nl (buf);
        if (!is_fileline (buf))
        {
            if (inside_comment == 0)
            {
                fprintf (cgiOut, "<b>Archive comment:</b>\n");
                inside_comment = 1;
            }
            if (inside_comment == 2)
                fprintf (cgiOut, "\n");
            fprintf (cgiOut, "%s\n", buf);
            continue;
        }
        else
        {
            if (inside_comment == 1)
                fprintf (cgiOut, "<hr>\n");
            inside_comment = 2;
        }
        
        l = strlen (buf);
        if (l > 1 && buf[l-1] == '/') continue; // skip directories
        
        buf[27] = '\0'; buf[42] = '\0'; buf[46] = '\0';
        buf[49] = '\0'; buf[52] = '\0'; buf[55] = '\0';
        
        f.type = TYPE_REGULAR; // it's file
        f.hs = 0;   // doesn't have any sense
        tm1.tm_sec = 0;   // not recorded in ZIP file
        tm1.tm_min = atoi (buf+53);
        tm1.tm_hour = atoi (buf+50);
        tm1.tm_mday = atoi (buf+40);
        tm1.tm_mon = mon2int (buf+43);
        tm1.tm_year = atoi (buf+47);
        tm1.tm_wday = 0;  // not needed anyway
        tm1.tm_yday = 0;  // not needed anyway
        tm1.tm_isdst = 0; // don't know what's that
        f.size = atoi (buf+19);
        strcpy (buf1, buf+56);
        f.name = buf1;
        
        fprintf (cgiOut, "%4d %3s %2d %02d:%02d %9ld ",
                1900+tm1.tm_year, month[tm1.tm_mon],
                tm1.tm_mday, tm1.tm_hour, tm1.tm_min, f.size);
        fprintf (cgiOut, "<a href=\""AUTOEXTRACT"?%s&archive=%s%s%s&file=%s\">%s</a>\n",
                put_options (0), hexify (path), hexify ("/"),
                hexify (name), hexify (f.name), f.name);
    }

    fprintf (cgiOut, "</pre>\n");
    fclose (fp);

    remove (tempfile1);
}

/*--------------------------------------------------------------------*/

int is_fileline (char *s)
{
    if (strspn (s, "drwxast-") < 7) return 0;
    return 1;
}

/*--------------------------------------------------------------------*/

void list_arj (char *path, char *name)
{
    char  buf[4096];

    fprintf (cgiOut, "<b>To prevent questions:</b>\n"
            "<br> -- extracting from ARJ archives isn't supported (yet). "
            "May be in some distant future;\n"
            "<br> -- no directories are shown in ARJ archive "
            "(UNARJ limitation);\n"
            "<br> -- no multivolume information is shown (UNARJ limitation).\n"
            "\n");
    
    // create listing by executing unarj
    fprintf (cgiOut, "\n<pre>\n");
    sprintf (buf, "unarj l %s/%s", make_absolute (path), name);
    fflush (stdout);
    cgiSaferSystem (buf);

    fprintf (cgiOut, "</pre>\n");
}

/*--------------------------------------------------------------------*/

void list_tar (char *path, char *name)
{
    int   l;
    char  buf[4096], buf1[4096], tempfile1[64], *p, *pp;
    FILE  *fp;
    struct file f;
    struct tm tm1;

    sprintf (tempfile1, TEMPDIR"contents.%d", getpid ());
    
    // create listing by executing tar
    sprintf (buf, "tar t%svf %s/%s >%s",
             mask_matchi ("*.tar", name) ? "" : "z",
             make_absolute (path), name, tempfile1);
    cgiSaferSystem (buf);

    // read and parse listing
    fp = fopen (tempfile1, "r");

    fprintf (cgiOut, "\n<pre>\n");
    while (fgets (buf, sizeof(buf), fp) != NULL)
    {
        strip_nl (buf);

        l = strlen (buf);
        if (l > 1 && buf[l-1] == '/') continue; // skip directories
        
        f.type = TYPE_REGULAR; // it's file
        f.hs = 0;   // doesn't have any sense
        tm1.tm_sec = 0;   // not recorded in TAR file
        pp = buf;
        if ((p = strsep1 (&pp, ' ')) == NULL) continue;  // left permissions
        if ((p = strsep1 (&pp, ' ')) == NULL) continue;  // left user/group
        if ((p = strsep1 (&pp, ' ')) == NULL) continue;  // left size
        f.size = atoi (p);
        if ((p = strsep1 (&pp, ' ')) == NULL) continue;  // left month
        tm1.tm_mon = mon2int (p);
        if ((p = strsep1 (&pp, ' ')) == NULL) continue;  // left day
        tm1.tm_mday = atoi (p);
        if ((p = strsep1 (&pp, ' ')) == NULL) continue;  // left time
        p[2] = '\0';
        tm1.tm_hour = atoi (p);
        tm1.tm_min = atoi (p+3);
        if ((p = strsep1 (&pp, ' ')) == NULL) continue;  // left year
        tm1.tm_year = atoi (p)-1900;
        
        strcpy (buf1, pp);
        f.name = buf1;
        
        tm1.tm_wday = 0;
        tm1.tm_yday = 0;
        tm1.tm_isdst = 0;
        //f.age = mktime ();
        
        fprintf (cgiOut, "%4d %3s %2d %02d:%02d %9ld ",
                1900+tm1.tm_year, month[tm1.tm_mon],
                tm1.tm_mday, tm1.tm_hour, tm1.tm_min, f.size);
        fprintf (cgiOut, "<a href=\""AUTOEXTRACT"?%s&archive=%s%s%s&file=%s\">%s</a>\n",
                put_options (0), hexify (path), hexify ("/"),
                hexify (name), hexify (f.name), f.name);
    }

    fprintf (cgiOut, "</pre>\n");
    fclose (fp);

    remove (tempfile1);
}
