#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <sys/fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <unistd.h>
#include <dirent.h>
#include <string.h>

#include "common.h"

/*--------------------------------------------------------------------*/

int current_depth, LI_ISSUED=0;

/*--------------------------------------------------------------------*/

static int process_directory (char *directory);
static int output_record (char *dirname, struct file *f);

/*--------------------------------------------------------------------*/

int cgiMain (void)
{
    int   i;
    
    get_options ();
    if (load_database ()) error ("Can't load database");

    cgiHeaderContentType ("text/html");
    
    fprintf (cgiOut, HTML_HEADER, "Directory tree");
    print_file (HEADER);

    fprintf (cgiOut, "<h2>Directory tree down to %d levels starting from %s</h2>\n\n",
             options.tree.max_depth, options.path);
    
    fprintf (cgiOut, "\n<center>"
             "<form method=get action=\""AUTOTREE"\">\n"
             "\n<table border=0 cellpadding=3 align=center>"
             "\n<tr>"
             "\n<td>Select expansion depth:\n"
             "<td><td>");
    for (i=1; i<=MAX_DEPTH; i++)
    {
        fprintf (cgiOut, "\n<input type=radio name=depth value=%d %s> %d",
                 i, options.tree.max_depth == i ? "checked" : "", i);
    }
    fprintf (cgiOut, "\n<td><input type=submit value=\"Display\">"
             "\n<tr>"
             "\n<td colspan=2><input type=checkbox name=opt6 value=1 %s> "
             "Show directory descriptions"
             "\n</table>"
             "\n<input type=hidden name=path value=\"%s\">",
             options.tree.print_desc ? "checked" : "",
             options.path);
    put_options_form (SKIP_TREE_DEPTH|SKIP_TREE_PRINTDESC|SKIP_PATH);
    fprintf (cgiOut, "\n\n</form>"
             "\n<br>Click on directory name to enter it"
             "\n<br>Click on "SIGN_EXPAND" near directory name to expand"
             " tree starting from that directory\n"
             "\n</center>\n");

    // divide path into separate directories
    print_directory_name (options.path, dir_description (options.path));
    fprintf (cgiOut, "<hr>\n");

    current_depth = 0;
    process_directory (options.path);

    print_footer ();
    print_file (FOOTER);
    print_file (COPYRIGHT);
    fprintf (cgiOut, HTTP_FOOTER);
    return 0;
}

/*--------------------------------------------------------------------*/

static int process_directory (char *directory)
{
    int               i, N, n;
    char              buf[2048];
    struct file       *f;

    current_depth++;
    
    f = (struct file *) malloc (MAX_ENTRIES * sizeof (struct file));
    N = 0;
    for (i=0; i<fb.N && N<MAX_ENTRIES-1; i++)
    {
        if (!strcmp (fb.files[i].dir, directory))
        {
            if (fb.files[i].type == TYPE_SYMLINK)
                n = resolve_link (fb.files[i]);
            else
                n = i;
            if (n < 0) continue;
            if (fb.files[n].type == TYPE_DIRECTORY)
                f[N++] = fb.files[i];
        }
    }
    qsort ((void *)f, N, sizeof (struct file), filecompare);

    // process directories
    if (N > 0) fprintf (cgiOut, "<ul>\n");
    LI_ISSUED = 0;
    for (i=0; i<N; i++)
    {
        output_record (directory, &(f[i]));
        if (current_depth < options.tree.max_depth)
        {
            if (f[i].type == TYPE_DIRECTORY)
            {
                strcpy (buf, directory);
                strcats (buf, f[i].name);
            }
            if (f[i].type == TYPE_SYMLINK)
            {
                n = resolve_link (f[i]);
                if (n < 0 || fb.files[n].type != TYPE_DIRECTORY) continue;
                strcpy (buf, fb.files[n].dir);
                strcats (buf, fb.files[n].name);
            }
            process_directory (buf);
        }
    }
    fprintf (cgiOut, "\n");
    free (f);
    current_depth--;
    if (N > 0) fprintf (cgiOut, "</ul>\n");
    return 0;
}

/*--------------------------------------------------------------------*/

static int output_record (char *dirname, struct file *f)
{
    char buf[4096];
    int  n;

    if (f->type == TYPE_SYMLINK)
    {
        n = resolve_link (*f);
        if (n < 0) return 0;
        strcpy (buf, fb.files[n].dir);
        strcats (buf, fb.files[n].name);
    }
    else
    {
        strcpy (buf, dirname);
        strcats (buf, f->name);
    }

    if (current_depth == options.tree.max_depth && LI_ISSUED)
        fprintf (cgiOut, " // ");
    else
    {
        fprintf (cgiOut, "\n<li>");
        LI_ISSUED = 1;
    }
    if (f->hs)
        fprintf (cgiOut, "\n<a href=\""AUTOINDEX"?%s&path=%s\">%s</a>&nbsp;"
                 "<a href=\""AUTOTREE"?%s&path=%s\">"SIGN_EXPAND"</a>",
                 put_options (SKIP_PATH), hexify (buf), f->name,
                 put_options (SKIP_PATH), hexify (buf));
    else
        fprintf (cgiOut, "\n<a href=\""AUTOINDEX"?%s&path=%s\">%s</a>",
                 put_options (SKIP_PATH), hexify (buf), f->name);
    if (options.tree.print_desc && f->desc != NULL)
        if (f->type == TYPE_SYMLINK)
        {
            if (fb.files[n].desc != NULL)
                fprintf (cgiOut, " %s", fb.files[n].desc);
            else
                fprintf (cgiOut, " -&gt;&nbsp;%s", f->desc);
        }
        else
            fprintf (cgiOut, " %s", f->desc);
            
    return 0;
}
