#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <stdarg.h>
#include <sys/fcntl.h>
#include <sys/time.h>
#include "common.h"

/*--------------------------------------------------------------------*/

struct __options options =
{ {DEFAULT_SEARCH, 0, 0, 1, 1}, {DEFAULT_SORT, 1, DEFAULT_SORT},
{DEFAULT_DEPTH, 1}, 0, 1, "/"};
char *month[12] = {
    "Jan", "Feb", "Mar", "Apr", "May", "Jun",
    "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};

/* ----------------------------------------------------------- */

int filecompare (__const__ void *elem1, __const__ void *elem2)
{
    struct file
        *f1 = (struct file *) elem1,
        *f2 = (struct file *) elem2;
    int  cn, cd;

    /*
     if (f1->type != f2->type) return (f2->type - f1->type);
     */
    
    if (options.sort.by_directory)
    {
        cd = strcmp (f1->dir,  f2->dir);
        if (cd) return cd;
    }
    cn = strcmp (f1->name, f2->name);

    switch (options.sort.type)
    {
    case 'n': return cn;
    case 'N': return -cn;
    case 's': return (f1->size != f2->size ? f2->size - f1->size : cn);
    case 'S': return (f2->size != f1->size ? f1->size - f2->size : cn);
    case 't': return (f1->age != f2->age ? f2->age - f1->age : cn);
    case 'T': return (f2->age != f1->age ? f1->age - f2->age : cn);
    }
    
    return 0;
}

/* ------------------------------------------------------------------------ */

int is_validsort (void)
{
    if (options.sort.type == 'n' || options.sort.type == 'N' ||
        options.sort.type == 's' || options.sort.type == 'S' ||
        options.sort.type == 't' || options.sort.type == 'T')
        return 1;
    return 0;
}

/*--------------------------------------------------------------------*/

void print_directory_name (char *dirname, char *desc)
{
    char buf1[4096], buf2[4096], *p1, *p2;
    int  done;
    
    // divide path into separate directories
    fprintf (cgiOut, "<hr>\n<b>%s</b> : ", desc == NULL ? "Directory of" : desc);
    fprintf (cgiOut, "<a href=\""AUTOINDEX"?%s&path=%s\">/</a>&nbsp;\n"
            "<a href=\""AUTOTREE"?%s&path=%s\">"SIGN_EXPAND"</a>\n",
            put_options (SKIP_PATH),
            hexify ("/"), put_options (SKIP_PATH), hexify ("/"));
    if (strcmp (dirname, "/") == 0) return;
    strcpy (buf1, dirname);
    p1 = buf1;
    done = 0;
    do
    {
        p2 = strchr (p1+1, '/');   /* p1, p2 now limit path component */
        if (p2 == NULL)
        {
            done = 1;
            p2 = strchr (p1+1, '\0');
            if (p2 == NULL) break;
        }
        *p2 = '\0';
        strcpy (buf2, dirname);
        buf2[p2-buf1] = '\0';
        fprintf (cgiOut, " / <a href=\""AUTOINDEX"?%s&path=%s\">%s</a>\n"
                "<a href=\""AUTOTREE"?%s&path=%s\">"SIGN_EXPAND"</a>\n",
                put_options (SKIP_PATH), hexify (buf2),
                p1+1, put_options (SKIP_PATH), hexify (buf2));
        p1 = p2;
    }
    while (!done);
}

/*--------------------------------------------------------------------*/

void print_sort_urls (char *url)
{
    // form those long and boring URLs for different sorting
    fprintf (cgiOut, "\n<hr>\nSort by: \n");
    
    if (options.sort.type == 'n')
        fprintf (cgiOut, "\n<b>name</b> ");
    else
        fprintf (cgiOut, "\n<a href=\"%s&sort=n\">name</a> ", url);
    
    if (options.sort.type == 'N')
        fprintf (cgiOut, "\n(<b>reversed</b>) | ");
    else
        fprintf (cgiOut, "\n(<a href=\"%s&sort=N\">reversed</a>) | ", url);
    
    if (options.sort.type == 's')
        fprintf (cgiOut, "\n<b>size</b> ");
    else
        fprintf (cgiOut, "\n<a href=\"%s&sort=s\">size</a> ", url);
    
    if (options.sort.type == 'S')
        fprintf (cgiOut, "\n(<b>reversed</b>) | ");
    else
        fprintf (cgiOut, "\n(<a href=\"%s&sort=S\">reversed</a>) | ", url);
    
    if (options.sort.type == 't')
        fprintf (cgiOut, "\n<b>date</b> ");
    else
        fprintf (cgiOut, "\n<a href=\"%s&sort=t\">date</a> ", url);
    
    if (options.sort.type == 'T')
        fprintf (cgiOut, "\n(<b>reversed</b>) | ");
    else
        fprintf (cgiOut, "\n(<a href=\"%s&sort=T\">reversed</a>)", url);
    
    fprintf (cgiOut, "\n");
}

/* --------------------------------------------------------------------
 prints sorted list of files in different directories
 */

void print_list (struct file *f, int N, int flags)
{
    char    *dp = NULL, buf[2048];
    int     i, view_opt, n;
    struct  file F;
    struct  tm tm1;

    // simple sanity check
    if (N <= 0) return;
    
    dp = NULL;
    for (i=0; i<N; i++)
    {
        if (f[i].type != TYPE_REGULAR && f[i].type != TYPE_SYMLINK) continue;
        if (f[i].type == TYPE_SYMLINK)
        {
            n = resolve_link (f[i]);
            if (n < 0 || fb.files[n].type != TYPE_REGULAR) continue;
            F = fb.files[n];
            /*
            F.dir  = f[i].dir;
            F.name = f[i].name;
            */
            if (F.desc == NULL)
            {
                sprintf (buf, "-&gt; %s", f[i].desc);
                F.desc = chunk_put (buf);
            }
        }
        else
            F = f[i];
        
        if (!(flags & LIST_SINGLE_DIR) &&
            options.sort.by_directory && (dp != F.dir))
        {
            fprintf (cgiOut, "\n\nDirectory <a href=\""AUTOINDEX"?%s&path=%s\">%s</a>",
                    put_options (SKIP_PATH), hexify (F.dir), F.dir);
            if (has_subdirs (F.dir))
                fprintf (cgiOut, "&nbsp;<a href=\""AUTOTREE"?%s&path=%s\">"SIGN_EXPAND"</a>",
                        put_options (SKIP_PATH), hexify (F.dir));
            dp = F.dir;
        }

        tm1 = *localtime (&(F.age));
        fprintf (cgiOut, "\n%4d %3s %2d %02d:%02d %9ld ",
                1900+tm1.tm_year, month[tm1.tm_mon],
                tm1.tm_mday, tm1.tm_hour, tm1.tm_min, F.size);

        if (options.group_selection)
        {
            strcpy (buf, F.dir);
            strcats (buf, F.name);
            fprintf (cgiOut, "<input type=checkbox name=d value=\"%s\" %s>", 
                    buf, (flags & LIST_ALL_CHECKED) ? "checked" : "");
            fprintf (cgiOut, "%s", F.name);
        }
        else
        {
            if (F.size >= HTTP_LIMIT)
                fprintf (cgiOut, "<a href=\"ftp://"FTP_SITE"%s/%s\">%s</a>",
                        F.dir, F.name, F.name);
            else
                fprintf (cgiOut, "<a href=\""WEB_ROOT"%s/%s\">%s</a>",
                        F.dir, F.name, F.name);
        }
                    
        view_opt = 0;
        if (is_viewable (F.name))
        {
            fprintf (cgiOut, " (<a href=\""WEB_ROOT"%s/%s\">"SIGN_VIEW"</a>",
                    F.dir, F.name);
            view_opt = 1;
        }
        if (is_listable (F.name))
        {
            if (!view_opt) fprintf (cgiOut, " (");
            else           fprintf (cgiOut, ", ");
            fprintf (cgiOut, "<a href=\""AUTOLIST"?%s&fpath=%s&file=%s\">"SIGN_CONTENTS"</a>",
                    put_options (0), hexify (F.dir), hexify (F.name));
            view_opt = 1;
        }
        if (view_opt) fprintf (cgiOut, ")");
        
        if (!(flags & LIST_SINGLE_DIR) &&
            options.search.print_where &&
            !options.sort.by_directory)
        {
            fprintf (cgiOut, "&nbsp;(<a href=\""AUTOINDEX"?%s&path=%s\">%s</a>)",
                    put_options (SKIP_PATH), hexify (F.dir), F.dir);
        }

        if (F.desc != NULL)
            fprintf (cgiOut, "\n                            <i>%s</i>", F.desc);
    }
}

/* --------------------------------------------------------------------*/

void get_options (void)
{
    char buf[2], buf1[8192];
    int  init, l;

    // path. we should try to query it before checking for init
    cgiFormStringNoNewlines ("path", options.path, sizeof (options.path));
    l = strlen (options.path);
    if (l > 1 && options.path[l-1] == '/') options.path[l-1] = '\0';
    if (options.path[0] == '\0') strcpy (options.path, "/");
    if (options.path[0] != '/')
    {
        strcpy (buf1, options.path);
        strcpy (options.path, "/");
        strcat (options.path, buf1);
    }
    
    // check for "init". if present, we should just set defaults
    cgiFormInteger ("init", &init, 0);
    
    // when no input is given, assume init=1
    if (cgiContentLength == 0) init = 1;
    
    if (init)
    {
        options.sort.type = options.sort.default_type;
        options.search.casesen = 0;
        options.search.exclude_dirnames = 0;
        options.sort.by_directory = 1;
        options.search.print_where = 1;
        options.search.from_subdir = 0;
        options.group_selection = 0;
        options.tree.print_desc = 1;
        options.tree.max_depth = DEFAULT_DEPTH;
        options.arc_method = 1;
        return;
    }
    
    // sort by what?
    options.sort.type = options.sort.default_type;
    cgiFormString ("sort", buf, sizeof (buf));
    if (buf[0] != '\0') options.sort.type = buf[0];
    if (!is_validsort ()) options.sort.type = options.sort.default_type;

    // search is case sensitive?
    cgiFormInteger ("opt1", &options.search.casesen, 0);

    // exclude directory names from search?
    cgiFormInteger ("opt2", &options.search.exclude_dirnames, 0);

    // sort by directory?
    cgiFormInteger ("opt3", &options.sort.by_directory, 0);

    // print directory where file is?
    cgiFormInteger ("opt4", &options.search.print_where, 0);

    // start search from certain subdirectory?
    cgiFormInteger ("opt5", &options.search.from_subdir, 0);

    // group selections?
    cgiFormInteger ("msel", &options.group_selection, 0);

    // print descriptions in tree?
    cgiFormInteger ("opt6", &options.tree.print_desc, 0);

    // tree depth
    cgiFormIntegerBounded ("depth", &options.tree.max_depth,
                           1, MAX_DEPTH, DEFAULT_DEPTH);
    // archiving method
    cgiFormIntegerBounded ("arcm", &options.arc_method,
                           1, 2, 1);
}

/* --------------------------------------------------------------------*/

char *put_options (int skip)
{
    static char opts[2048];
    char        buf[256];
    int         l;

    opts[0] = '\0';
    
    // sort by what?
    if (!(skip & SKIP_SORT_TYPE) && options.sort.type != DEFAULT_SORT)
    {
        sprintf (buf, "sort=%c&", options.sort.type);
        strcat (opts, buf);
    }

    // search is case sensitive?
    if (!(skip & SKIP_SRCH_CASESEN) && options.search.casesen)
        strcat (opts, "opt1=1&");

    // exclude directory names from search?
    if (!(skip & SKIP_SRCH_NODIRS) && options.search.exclude_dirnames)
        strcat (opts, "opt2=1&");

    // sort by directory?
    if (!(skip & SKIP_SORT_BYDIR) && options.sort.by_directory)
        strcat (opts, "opt3=1&");

    // print directory where file is?
    if (!(skip & SKIP_SRCH_PRINTWHERE) && options.search.print_where)
        strcat (opts, "opt4=1&");

    // start search from certain subdirectory?
    if (!(skip & SKIP_SRCH_FROMSUBDIR) && options.search.from_subdir)
        strcat (opts, "opt5=1&");
    
    // group selections?
    if (!(skip & SKIP_GROUPSEL) && options.group_selection)
        strcat (opts, "msel=1&");

    // print descriptions in tree?
    if (!(skip & SKIP_TREE_PRINTDESC) && options.tree.print_desc)
        strcat (opts, "opt6=1&");
    
    // tree depth
    if (!(skip & SKIP_TREE_DEPTH) && options.tree.max_depth != DEFAULT_DEPTH)
    {
        sprintf (buf, "depth=%d&", options.tree.max_depth);
        strcat (opts, buf);
    }

    // archiving method
    if (!(skip & SKIP_ARCMETHOD) && options.arc_method != 1)
    {
        sprintf (buf, "arcm=%d&", options.arc_method);
        strcat (opts, buf);
    }

    // path
    if (!(skip & SKIP_PATH))
    {
        sprintf (buf, "path=%s&", hexify (options.path));
        strcat (opts, buf);
    }

    // remove ending &
    l = strlen (opts);
    if (l > 1 && opts[l-1] == '&') opts[l-1] = '\0';

    return opts;
}

/* --------------------------------------------------------------------*/

void put_options_form (int skip)
{
    // sort by what?
    if (!(skip & SKIP_SORT_TYPE) && options.sort.type != DEFAULT_SORT)
        fprintf (cgiOut, "\n<input type=hidden name=sort value=%c>", options.sort.type);

    // search is case sensitive?
    if (!(skip & SKIP_SRCH_CASESEN) && options.search.casesen)
        fprintf (cgiOut, "\n<input type=hidden name=opt1 value=1>");

    // exclude directory names from search?
    if (!(skip & SKIP_SRCH_NODIRS) && options.search.exclude_dirnames)
        fprintf (cgiOut, "\n<input type=hidden name=opt2 value=1>");

    // sort by directory?
    if (!(skip & SKIP_SORT_BYDIR) && options.sort.by_directory)
        fprintf (cgiOut, "\n<input type=hidden name=opt3 value=1>");

    // print directory where file is?
    if (!(skip & SKIP_SRCH_PRINTWHERE) && options.search.print_where)
        fprintf (cgiOut, "\n<input type=hidden name=opt4 value=1>");

    // start search from certain subdirectory?
    if (!(skip & SKIP_SRCH_FROMSUBDIR) && options.search.from_subdir)
        fprintf (cgiOut, "\n<input type=hidden name=opt5 value=1>");
    
    // group selections?
    if (!(skip & SKIP_GROUPSEL) && options.group_selection)
        fprintf (cgiOut, "\n<input type=hidden name=msel value=1>");

    // print descriptions in tree?
    if (!(skip & SKIP_TREE_PRINTDESC) && options.tree.print_desc)
        fprintf (cgiOut, "\n<input type=hidden name=opt6 value=1>");
    
    // tree depth 
    if (!(skip & SKIP_TREE_DEPTH) && options.tree.max_depth != DEFAULT_DEPTH)
        fprintf (cgiOut, "\n<input type=hidden name=depth value=%d>",
                 options.tree.max_depth);
    
    // archiving method
    if (!(skip & SKIP_ARCMETHOD) && options.arc_method != 1)
        fprintf (cgiOut, "\n<input type=hidden name=arcm value=%d>",
                 options.arc_method);

    // path
    if (!(skip & SKIP_PATH))
        fprintf (cgiOut, "\n<input type=hidden name=path value=\"%s\">",
                 options.path);
}

/* --------------------------------------------------------------------*/

void print_footer (void)
{
    fprintf (cgiOut, "\n<hr>"
             "\n"
             "\n<center>"
             "\nYou are now in the <b>[%s]</b> directory<br>"
             "\n<b>"
             "\n["
             "\n<a href=\""ARCHIVE_URL"index.html\">Archive home</a> |"
             "\n<a href=\""ARCHIVE_URL"shortcuts.html\">Shortcuts</a> |"
             "\n<a href=\""AUTOINDEX"?%s\">Files</a> |"
             "\n<a href=\""AUTOTREE"?%s\">Directory tree</a> |"
             "\n<a href=\""AUTONEWFILES"?%s\">New files</a> |"
             "\n<a href=\""AUTOSEARCH"?%s\">Search</a> |"
             "\n<a href=\""ARCHIVE_URL"help.html\">Help</a>"
             "\n]"
             "\n</b></center>"
             "\n",
             options.path,
             put_options (0), put_options (0), put_options (0),
             put_options (SKIP_SORT_TYPE));
}

/* ------------------------------------------------------------------ */

int in_the_search_scope (int n, char *startpath)
{
    if (search_ignore (fb.files[n].name)) return 0;
    if (!options.search.from_subdir) return 1;
    return !strncmp (fb.files[n].dir, startpath, strlen (startpath));
}

/* ------------------------------------------------------------------ */

void error (char *format, ...)
{
    va_list    args;

    cgiHeaderContentType ("text/html");
    fprintf (cgiOut, HTML_HEADER, "error while processing request");
    print_file (HEADER);
    
    fprintf (cgiOut, "\n<h1>Error</h1>");

    va_start (args, format);
    vfprintf (cgiOut, format, args);
    va_end (args);
    
    print_footer ();
    print_file (FOOTER);
    print_file (COPYRIGHT);
    fprintf (cgiOut, HTTP_FOOTER);
    fflush (cgiOut);
    exit (1);
}

