#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/fcntl.h>
#include <sys/stat.h>

#include "tools.h"
#include "onsite.h"
#include "filebase.h"

struct filedata1
{
    struct file fd;
    int         dir_offset;    // offset in directory file
    int         name_offset;   // offset in name file
    int         desc_offset;   // offset in description file (if available)
};

/*--------------------------------------------------------------------*/

static int   fh_main, fh_name, fh_desc;
static int   name_off=0, desc_off=0;
static char  *names, *descriptions;

struct __fb fb;

/*--------------------------------------------------------------------*/

int load_filelist (void)
{
    struct filedata1   *files1;
    int  i;
    long L;
    
    // load files into memory
    if (load_file (NAMELIST, &names) < 0) return 1;
    if (load_file (DESCLIST, &descriptions) < 0) return 1;
    if ((L=load_file (FILELIST, (char **) &files1)) < 0) return 1;

    // make 'files' from 'files1'
    fb.N = L / sizeof (struct filedata1);
    fb.files = malloc (fb.N * sizeof (struct filedata));
    for (i=0; i<fb.N; i++)
    {
        fb.files[i].fd = files1[i].fd;
        fb.files[i].dir  = names + files1[i].dir_offset;
        fb.files[i].name = names + files1[i].name_offset;
        if (files1[i].desc_offset != 0)
            fb.files[i].desc = descriptions + files1[i].desc_offset;
        else
            fb.files[i].desc = NULL;
    }
    free (files1);
    fb.timestamp = * ((time_t *) (descriptions + 2));
    return 0;
}


/*--------------------------------------------------------------------*/

int create_filebase (void)
{
    time_t  now;
    
    fh_main = open (FILELIST, O_CREAT|O_TRUNC|O_RDWR, S_IREAD|S_IWRITE);
    fh_name = open (NAMELIST, O_CREAT|O_TRUNC|O_RDWR, S_IREAD|S_IWRITE);
    fh_desc = open (DESCLIST, O_CREAT|O_TRUNC|O_RDWR, S_IREAD|S_IWRITE);
    if (fh_main < 0 || fh_name < 0 || fh_desc < 0) return 1;

    // empty description and timestamp
    write (fh_desc, "$", 2);
    desc_off += 2;
    now = time (NULL);
    write (fh_desc, &now, sizeof (time_t));
    desc_off += sizeof (time_t);
    return 0;
}

/*--------------------------------------------------------------------*/

int close_filebase (void)
{
    close (fh_main);
    close (fh_name);
    close (fh_desc);
    return 0;
}

/*--------------------------------------------------------------------*/

int write_name (char *txt)
{
    int  l = strlen (txt)+1;
    
    if (write (fh_name, txt, l) != l) return -1;
    name_off += l;
    return name_off-l;
}

/*--------------------------------------------------------------------*/

int write_desc (char *txt)
{
    int  l = strlen (txt)+1;
    
    if (write (fh_desc, txt, l) != l) return -1;
    desc_off += l;
    return desc_off-l;
}

/*--------------------------------------------------------------------*/

int write_item (struct filedata *f, int dr_o, int nm_o, int ds_o)
{
    struct filedata1 f1;
    
    f1.fd = f->fd;
    f1.dir_offset  = dr_o;
    f1.name_offset = nm_o;
    f1.desc_offset = ds_o;
    if (write (fh_main, &f1, sizeof (f1)) != sizeof (f1)) return -1;
    return 0;
}
