
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/fcntl.h>

#include "tools1.h"

static int try (char *t, char *s);

static char *month[12] = {
    "Jan", "Feb", "Mar", "Apr", "May", "Jun",
    "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};

#define max(a,b) (((a) > (b)) ? (a) : (b))
#define min(a,b) (((a) < (b)) ? (a) : (b))
#define split_to_digits(n,i0,i1,i2) {i0 = n/100; i1 = (n-i0*100)/10; \
    i2 = (n-i0*100-i1*10);}

/*--------------------------------------------------------------------*/

long file_length (int fh)
{
    unsigned long L;
    L = lseek (fh, 0, SEEK_END);
    lseek (fh, 0, SEEK_SET);
    return L;
}

/*--------------------------------------------------------------------*/

char *stristr (char *haystack, char *needle)
{
    int   lh = strlen (haystack), ln = strlen (needle);
    char  *b = malloc (lh+ln+2), *ph = b, *pn = b + lh + 1, *p;
    
    if (b == NULL) return NULL;
    
    strcpy (ph, haystack);
    strcpy (pn, needle);
    p = ph;
    while (*p) *p++ = tolower (*p);
    p = pn;
    while (*p) *p++ = tolower (*p);
    p = strstr (ph, pn);
    free (b);
    
    if (p == NULL) return NULL;
    return haystack + (p - ph);
}

/*--------------------------------------------------------------------*/

int mask_matchi (char *template, char *sample)
{
    int   lt = strlen (template), ls = strlen (sample), result;
    char  *b = malloc (lt+ls+2), *pt = b, *ps = b + lt + 1, *p;
    
    if (b == NULL) return 0;
    
    strcpy (pt, template);
    strcpy (ps, sample);
    p = pt;
    while (*p) *p++ = tolower (*p);
    p = ps;
    while (*p) *p++ = tolower (*p);
    
    result = mask_match (pt, ps);
    free (b);
    return result;
}

/* -----------------------------------------------------------
 list of viewable extensions and files:
 */

int is_viewable (char *filename)
{
    if (mask_matchi ("*.tar.*", filename)) return 0;
    if (mask_matchi ("*.tar", filename)) return 0;
    if (mask_matchi ("*.tgz", filename)) return 0;
    if (mask_matchi ("*.gz", filename)) return 0;
    if (mask_matchi ("*.zip", filename)) return 0;
    if (mask_matchi ("*.arj", filename)) return 0;
    if (mask_matchi ("*.lh?", filename)) return 0;
    if (mask_matchi ("*.exe", filename)) return 0;
    if (mask_matchi ("*.ha", filename)) return 0;
    if (mask_matchi ("*.wav", filename)) return 0;
    if (mask_matchi ("*.mid", filename)) return 0;
    if (mask_matchi ("*.au", filename)) return 0;
    if (mask_matchi ("*.z", filename)) return 0;
    if (mask_matchi ("*.hqx", filename)) return 0;
    return 1;
}

/* ----------------------------------------------------------- */

int is_listable (char *filename)
{
    if (mask_matchi ("*.tar", filename)) return 1;
    if (mask_matchi ("*.taz", filename)) return 1;
    if (mask_matchi ("*.tgz", filename)) return 1;
    if (mask_matchi ("*.tar.z", filename)) return 1;
    if (mask_matchi ("*.tar.gz", filename)) return 1;
    if (mask_matchi ("*.zip", filename)) return 1;
    if (mask_matchi ("*.exe", filename)) return 1;
    if (mask_matchi ("*.arj", filename)) return 1;
    return 0;
}

/* ----------------------------------------------------------- */

int ignore (char *filename)
{
    if (!strcmp (".", filename)) return 1;
    if (!strcmp ("..", filename)) return 1;
    if (!strcmp ("00-index", filename)) return 1;
    if (!strcmp ("00index.txt", filename)) return 1;
    if (!strcmp ("00indexd.txt", filename)) return 1;
    if (!strcmp ("00global.txt", filename)) return 1;
    if (!strcmp ("00globald.txt", filename)) return 1;
    if (!strcmp (".description", filename)) return 1;
    return 0;
}

/* ----------------------------------------------------------- */

int search_ignore (char *filename)
{
    if (!strcmp ("CHECKSUMS", filename)) return 1;
    if (!strcmp (".mirror", filename)) return 1;
    return 0;
}

/* ------------------------------------------------------------------------ */

int mask_match (char *template, char *sample)
{
    return try (template, sample);
}

/* --------------------------------------------------------------------- */

static int try (char *t, char *s)
{
    char *tp = t, *sp = s;
    
    if (strlen (t) == 1 && *t == '*') return 1;
    
    //printf ("try: [%s] - [%s]\n", t, s);
    while (*tp && *sp)
    {
        //printf ("tp = [%s], sp = [%s]\n", tp, sp);
        if (*tp == '*')
        {
            do
            {
                sp++;
                if (try (tp+1, sp)) return 1;
            }
            while (*sp);
            return 0;
        }
        else if (*tp == '?')
        {
            tp++; sp++;
        }
        else if (*tp != *sp) 
        {
            return 0;
        }
        else
        {
            tp++; sp++;
        }
    }
    if (*sp || (*tp && (*tp != '*' || strlen(tp) > 1)))
        /*if (*tp || *sp) */
        return 0;
    else
        return 1;
}

/* ------------------------------------------------------------- */

char *insert_commas (unsigned long number)
{
    static char      buf[16];
    unsigned long    i00, i01, i02;
    unsigned long    i0, i1, i2;
    
    number = min (999999999L, number);
    
    i00 = number/1000000;
    i01 = (number-i00*1000000)/1000;
    i02 = (number-i00*1000000-i01*1000);
    
    if (i00 != 0)
    {
        split_to_digits (i00, i0, i1, i2);
        buf[0] = (i0==0) ? ' ' : '0'+i0;
        buf[1] = (i1+i0==0) ? ' ' : '0'+i1;
        buf[2] = '0'+i2;
        buf[3] = ',';
    }
    else
        memset (buf, ' ', 4);
    
    if (i01+i00 != 0)
    {
        split_to_digits (i01, i0, i1, i2);
        if (i00 != 0)
        {
            buf[4] = '0'+i0;
            buf[5] = '0'+i1;
            buf[6] = '0'+i2;
        }
        else
        {
            buf[4] = (i0==0) ? ' ' : '0'+i0;
            buf[5] = (i1+i0==0) ? ' ' : '0'+i1;
            buf[6] = '0'+i2;
        }
        buf[7] = ',';
    }
    else
        memset (buf+4, ' ', 4);
    
    split_to_digits (i02, i0, i1, i2);
    if (i01+i00 != 0)
    {
        buf[8]  = '0'+i0;
        buf[9]  = '0'+i1;
        buf[10] = '0'+i2;
    }
    else
    {
        buf[8]  = (i0==0) ? ' ' : '0'+i0;
        buf[9]  = (i1+i0==0) ? ' ' : '0'+i1;
        buf[10] = '0'+i2;
    }
    buf[11] = '\0';
    
    return buf;
}

/* ------------------------------------------------------------- */

char *chunk_put (char *s)
{
    /*
    int   len = strlen (s);
    char  *p;
    
    if (chunk == NULL || fchunk-chunk-3 < len)
    {
        chunk = malloc (CHUNK_SIZE);
        fchunk = chunk;
    }
    
    p = fchunk;
    strcpy (p, s);
    fchunk += len+1;
    
    return p;
    */
    
    return strcpy (malloc (strlen(s)+1), s);
}

/* ------------------------------------------------------------- */

void print_file (char *filename)
{
    FILE  *fp;
    char  buf1[8192];
    
    // read file line by line and print it
    fp = fopen (filename, "rt");
    if (fp == NULL) return;
    while (fgets (buf1, sizeof(buf1), fp) != NULL) fputs (buf1, stdout);
    fclose (fp);
}

/*--------------------------------------------------------------------*/

char *strcats (char *dest, char *src)
{
    int l = strlen (dest);
    if (l > 1 && dest[l-1] != '/')
        strcat (dest, "/");
    if (strcmp (src, "/") != 0) strcat (dest, src);

    l = strlen (dest);
    if (l > 1 && dest[l-1] == '/')
        dest[l-1] = '\0';

    return dest;
}

/*--------------------------------------------------------------------*/

int mon2int (char *mon)
{
    int  i;
    
    for (i=0; i<12; i++)
        if (strcmp (mon, month[i]) == 0)
            return i;
    return 0;
}

/*--------------------------------------------------------------------*/

void strip_nl (char *s)
{
    int l;

    while ((l = strlen (s)) > 1 &&
           (s[l-1] == '\n' || s[l-1] == '\r'))
    {
        s[l-1] = '\0';
    }
}

/*--------------------------------------------------------------------*/

char *strsep1 (char **s, char d)
{
    char *p, *p1;

    // skip delimiters before meaningful characters
    p = *s;
    while (*p == d && *p) p++;
    if (*p == '\0') return NULL;

    // find next delimiter. p now points to wanted token
    p1 = strchr (p, d);
    if (p1 == NULL) return NULL;

    // found. put delimiting 0x00 and return pointer to token
    *p1 = '\0';
    *s = p1 + 1;
    return p;
}

/*--------------------------------------------------------------------*/

static char unsafe[] = ";/?:@&=+ \"#%<>";

char *hexify (char *src)
{
    char *b = malloc (strlen(src)*3+1), *p1 = src, *p2 = b;

    while (*p1)
    {
        if (*p1 < ' ' || *p1 > '~' || strchr (unsafe, *p1) != NULL)
        {
            sprintf (p2, "%%%02X", *p1);
            p2 += 3;
        }
        else
        {
            *p2 = *p1;
            p2++;
        }
        p1++;
    }
    *p2 = '\0';
    return b;
}

/*--------------------------------------------------------------------*/

long load_file (char *filename, char **p)
{
    int fh;
    long l;
    
    // open file
    fh = open (filename, O_RDONLY);
    if (fh < 0) return -1;
    
    // allocate buffer
    l = file_length (fh);
    if (l <= 0) {close (fh); return -1;}
    *p = malloc (l);
    if (*p == NULL) {close (fh); return -1;}

    // read file into buffer
    if (read (fh, *p, l) != l) {free (*p); close (fh); return -1;}
    
    close (fh);
    return l;
}

/*--------------------------------------------------------------------*/

char *concatenate_paths (char *root, char *branch)
{
    static char buffer[8192];
    char  *p, *p1, buf[8192];

    if (branch[0] == '/')
    {
        strcpy (buffer, branch);
        return buffer;
    }
    
    strcpy (buffer, root);
    strcats (buffer, branch);
    
    while ((p = strstr (buffer, "/..")) != NULL)
    {
        strcpy (buf, p+3);
        *p = '\0';
        p1 = strrchr (buffer, '/');
        if (p1 == NULL)
        {
            strcpy (p, "/");
            strcat (p, buffer);
        }
        else
        {
            strcpy (p1, buf);
        }
    }
    return buffer;
}

/*--------------------------------------------------------------------*/

char *tempfile (void)
{
    static char buffer[4096];
    static int  N = 0;

    do
    {
        sprintf (buffer, "/tmp/auto.tmp.%d.%d", getpid (), N);
        N++;
    }
    while (access (buffer, F_OK) == 0);
    return buffer;
}

/* ------------------------------------------------------------- */

void translate (char *s, char in, char out)
{
    while (*s)
    {
        if (*s == in) *s = out;
        s++;
    }
}

/* ------------------------------------------------------------- */

void squeeze_path (char *path)
{
    char *p, *p1, *buf = malloc(strlen(path)+3);
    int  head_slash = 0, tail_slash = 0;

    if (path[0] == '\0') return;

    // check original path for starting/ending slashes
    if (path[0] == '/') head_slash = 1;
    if (path[strlen(path)-1] == '/') tail_slash = 1;
    
    if (head_slash)
        strcpy (buf, path);
    else
    {
        strcpy (buf, "/");
        strcat (buf, path);
    }
    if (!tail_slash)
        strcat (buf, "/");
    
    // remove '..'s from path
    while ((p = strstr (buf, "/../")) != NULL)
    {
        *p = '\0';
        p1 = strrchr (buf, '/');
        if (p1 == NULL)
            strcpy (buf, p+3);
        else
            strcpy (p1, p+3);
    }

    // squeeze subsequent slashes into one
    while ((p = strstr (buf, "//")) != NULL)
    {
        *p = '\0';
        strcpy (p, p+1);
    }
    
    // squeeze '/./'s
    while ((p = strstr (buf, "/./")) != NULL)
    {
        *p = '\0';
        strcpy (p, p+2);
    }

    path[0] = '\0';
    if (head_slash)
        strcat (path, "/");
    
    if (buf[0] == '/')
        strcat (path, buf+1);
    else
        strcat (path, buf);

    free (buf);
    
    // root directory is a special case
    if (strlen(path) == 1)
        return;

    // attach trailing / if needed
    if (path[strlen(path)-1] == '/')
        path[strlen(path)-1] = '\0';
    if (tail_slash)
        strcat (path, "/");
    
}
